"""DateTime Argument Types."""
import datetime
from typing import Any

from .argument import Argument


class DateTimeArgument(Argument):
    """Handle datetime creation."""

    def _parse_value(self, arg_value: str) -> Any:
        """Parse the argument value itself."""
        try:
            return datetime.datetime.fromisoformat(arg_value)
        except ValueError:
            raise TypeError(f"Supplied value to argument {self.label} is of incorrect format.")


class DateArgument(Argument):
    """Handle date creation."""

    def _parse_value(self, arg_value: str) -> Any:
        """Parse the argument value itself."""
        try:
            return datetime.date.fromisoformat(arg_value)
        except ValueError:
            raise TypeError(f"Supplied value to argument {self.label} is of incorrect format.")
