"""Command hooks."""
from typing import TYPE_CHECKING

from epplib.commands import CreateContact, UpdateContact
from epplib.commands.list import GetResults

from eppic.models import ExecutableCommand

if TYPE_CHECKING:
    from eppic.app import Application


def get_results_hook(app: 'Application', epp_command: ExecutableCommand) -> None:
    """Print results immidiately after list commands.

    This hook doesn't run if `app.settings.print_list_results_automatically` is `False`.
    """
    if app.settings.print_list_results_automatically:
        app.execute(GetResults())


def apply_disclose_policy(app: 'Application', epp_command: ExecutableCommand) -> None:
    """Apply the disclose policy according to the `epp_handler`'s.

    Requires `epp_command` to have a `disclose` field.
    """
    assert isinstance(epp_command, (CreateContact, UpdateContact))  # nosec

    policy = app.epp_handler.policy
    if policy is not None and epp_command.disclose is not None:
        epp_command.disclose = policy.apply_disclose_policy(epp_command.disclose)
