from unittest import TestCase
from unittest.mock import Mock, sentinel

from epplib.commands.list import GetResults
from epplib.commands.update import UpdateContact

from eppic.app import Application
from eppic.commands.hooks import apply_disclose_policy, get_results_hook

from .utils import DummyEppCommand, GetAppMixin


class TestGetResultsHook(GetAppMixin, TestCase):

    def _prep_get_results(self, enabled=True):
        settings = self.get_settings(confirm=True)
        settings.print_list_results_automatically = enabled
        app = Application(settings, "foobar asdf")
        return app

    def test_automatically_print_results(self):
        app = self._prep_get_results(True)
        get_results_hook(app, Mock())
        self.epp_handler.send_command.assert_called_once()
        # check that it was called with the `GetResults` command instance.
        arg = self.epp_handler.send_command.call_args[0][0]
        self.assertIsInstance(arg, GetResults)

    def test_automatically_print_results_disabled(self):
        app = self._prep_get_results(False)
        get_results_hook(app, Mock())
        self.epp_handler.send_command.assert_not_called()


class TestDisclosePolicy(GetAppMixin, TestCase):

    def setUp(self):
        super().setUp()
        self.policy = Mock()
        self.epp_handler.policy = self.policy
        self.disclosed_command = UpdateContact

    def test_apply_disclose_wrong_command(self):
        self.assertRaises(AssertionError, apply_disclose_policy, self.get_app(), DummyEppCommand("foobar"))

    def test_apply_disclose_disclose_is_none(self):
        apply_disclose_policy(self.get_app(), self.disclosed_command("asdf"))
        self.policy.apply_disclose_policy.assert_not_called()

    def test_apply_disclose_correct(self):
        apply_disclose_policy(self.get_app(), self.disclosed_command("asdf", disclose=sentinel.disclose))
        self.policy.apply_disclose_policy.assert_called_once_with(sentinel.disclose)
