from dataclasses import dataclass
from typing import Iterable, Optional, cast
from unittest import TestCase
from unittest.mock import Mock

from epplib.commands.base import Command as EppCommand

from eppic.commands.command import Command
from eppic.models import ArgumentField


@dataclass
class DummyEppCommand(EppCommand):
    foo: str
    bar: int

    def _get_command_payload(self) -> None:  # pragma: no cover
        # Dummy implementation of abstract abstract method.
        pass


class TestCommand(TestCase):
    """Test Command class."""

    def setUp(self):
        self.command_class_mock = DummyEppCommand

    def command(self, extensions: Optional[Iterable[ArgumentField]] = None) -> Command:
        return Command(
            "foobar",
            self.command_class_mock,
            (ArgumentField("foo", str, True), ArgumentField("bar", int, True)),
            Mock(),
            extensions=extensions
        )

    def test_pop_extensions(self):
        cmd = self.command([ArgumentField("asdf", int, False)])
        args = {"foo": "foo", "asdf": 69}

        self.assertEqual(cmd._pop_extensions(args), [69])
        self.assertEqual(args, {"foo": "foo"})

    def test_pop_extensions_no_extensions(self):
        cmd = self.command([ArgumentField("asdf", int, False)])
        args = {"foo": "foo", "asdf": None}

        self.assertEqual(cmd._pop_extensions(args), [])
        self.assertEqual(args, {"foo": "foo"})

    def test_generate(self):
        cmd = self.command()
        args = {"foo": "foo", "bar": 69}

        cmd_class = cast(EppCommand, cmd.prepare_command(args))
        self.assertEqual(cmd_class, DummyEppCommand("foo", 69))
        self.assertEqual(cmd_class.extensions, [])

    def test_generate_w_extensions(self):
        cmd = self.command([ArgumentField("baz", str, False)])
        args = {"foo": "foo", "bar": 69, "baz": "hello"}

        self.assertEqual(cast(EppCommand, cmd.prepare_command(args)).extensions, ["hello"])

    def test_generate_w_extensions_not_allowed(self):
        cmd = self.command([ArgumentField("baz", str, False)])
        cmd._command_class = Mock()
        args = {"foo": "foo", "bar": 69, "baz": "hello"}

        self.assertEqual(cmd.prepare_command(args), cast(Mock, cmd._command_class).return_value)
        cast(Mock, cmd._command_class).return_value.add_extension.assert_not_called()
